package com.github.polpetta.mezzotre.telegram.command;

import com.github.polpetta.mezzotre.i18n.TemplateContentGenerator;
import com.github.polpetta.mezzotre.orm.model.TgChat;
import com.google.inject.assistedinject.Assisted;
import com.pengrad.telegrambot.model.Update;
import com.pengrad.telegrambot.request.BaseRequest;
import com.pengrad.telegrambot.request.SendMessage;
import java.util.Collections;
import java.util.Optional;
import java.util.Set;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.Executor;
import javax.inject.Inject;
import javax.inject.Named;

/**
 * Generates a "Command not found" message
 *
 * @author Davide Polonio
 * @since 1.0
 * @see com.github.polpetta.mezzotre.telegram.callbackquery.NotFound for the event-based version
 */
class NotFound implements Processor {
  private static final String CMD_CTX_NAME = "cmdName";

  private final String commandName;
  private final TemplateContentGenerator templateContentGenerator;
  private final Executor threadPool;

  @Inject
  public NotFound(
      @Assisted String commandName,
      TemplateContentGenerator templateContentGenerator,
      @Named("eventThreadPool") Executor threadPool) {
    this.commandName = commandName;
    this.templateContentGenerator = templateContentGenerator;
    this.threadPool = threadPool;
  }

  @Override
  public Set<String> getTriggerKeywords() {
    return Collections.emptySet();
  }

  @Override
  public CompletableFuture<Optional<BaseRequest<?, ?>>> process(TgChat chat, Update update) {
    return CompletableFuture.supplyAsync(
            () ->
                templateContentGenerator.mergeTemplate(
                    ctx -> ctx.put(CMD_CTX_NAME, commandName),
                    chat.getLocale(),
                    "template/telegram/notFound.vm"),
            threadPool)
        .thenApply(text -> Optional.of(new SendMessage(chat.getId(), text)));
  }
}
